unit AniMapUploaderApi;
uses DebugHelperUnit, simplexml, AniMapUploaderSettings;

 { I DebugHelper.lib.pas} //ladc knihovna
 {$I groundspeak.config.pas} //nick

type
  TApiResult = record
    error: string;
    result: string;
  end;

  //pelo error zskanej z API
  function TranslateAPIError(Error: string): string;
  begin
    Result := _('Unknown error happened while working with server. Please try it later.');
    case Error of
      'ERR_CONNECTION':         Result := _('Error while connecting to the web side of project!');
      'ERR_REG_AGREE' :         Result := _('You must agree with terms of use!');
      'ERR_REG_EMAIL_INVALID' : Result := _('Entered email is not valid email address!');
      'ERR_REG_PASS_SHORT' :    Result := _('Entered password is too short. It must have at least 5 characters.');
      'ERR_REG_EMAIL_EXIST' :   Result := _('Entered email is registered yet. Try "Change Password" opinion or choose another email.');
      'ERR_TOKEN_EMAIL' :       Result := _('Error happened while generating token. Please contact author at vaclav@valicek.name');
      'ERR_LOG_EMAIL_EXIST' :   Result := _('Error - entered email isn`t registered already!');
      'ERR_LOG_PASS' :          Result := _('Error - wrong password!');
      'ERR_LOG_COUNT' :         Result := _('Error - you have tried to log in too much. Please wait 10 minutes before next try!');
      'ERR_PASS_EMAIL_EXIST' :  Result := _('Entered email isn`t in out DB. Please use "Register" button.');
      'ERR_AUTH' :              Result := _('Error while getting upload key!');
      //Soubor neexisuje
      'ERR_NO_FILE':            Result := _('AniMapUploader tried file which doesn`t exist. Failed ...');
      //UPLOAD
      'ERR_NO_AUTH' :           Result := _('Session probbably expired.');
      'ERR_BAD_EXT' :           Result := _('Filename extension is not .gif');
      'ERR_BAD_FILENAME':       Result := _('Filename is too short or contains unuseable characters.');
      'ERR_NO_DATA' :           Result := _('No data sent!');
      'ERR_NOT_GIF':            Result := _('Sent data aren`t gif!');
      'ERR_UID' :               Result := _('Bad UID. Please, contact author.');
      //'' : Result := _('');
    end;
  end;

  //zavol metodu api se zadanmi parametry a vytvo z n XML objekt
  function ApiRequest(method: String; GET: String): TJclSimpleXML;
  var
    xmlString: string;
  begin
    //debug (parametry)
    LDH('ApiRequest(' + method + ', ' + GET + ')');
    LDH('XML.Create');
    //vytvom xml soubor
    Result := TJclSimpleXML.Create;
    //sthnu data
    HttpAsk('GET', 'http://am.valicek.name/api/' + method + '.xml' + GET, '', '', xmlString);
    //pozor na chyby bez xml zabalen
    if (xmlString = 'ERR_BAD_FORMAT') or (xmlString = '') or (xmlString = 'ERR_METHOD') then exit;
    //vypu XML string
    LDH(xmlString);
    try
      Result.LoadFromString(xmlString);
      LDH('ROOT: ' + Result.Root.Name);
    finally
      //free probhne mimo tuto funkci
    end;
  end;


  Function ApiResult(XML: TjclSimpleXML): TApiResult;
  var
    I: integer;
  begin
    //pedpokldm, e error nastal - nenaetl se korektn soubor
    Result.error := 'ERR_CONNECTION';
    Result.result := '';
    //pokud nen root callback, mm smlu a patnej soubor
    if XML.Root.Name <> 'callback' then exit;
    //pokud jsem nedostal nic
    if XML.Root.Items.count < 1 then exit;
    //pro kadej prvek XML souboru
    for i  := 0 to XML.Root.Items.count - 1 do
    begin
      //s polokou
      with XML.Root.Items[i] do
      begin
        //pokud je to error!
        if Name = 'error' then begin Result.error := Value; exit; end;
        if Name = 'result' then Result.result := Value;
      end;
    end;
    if (Result.Error = 'ERR_CONNECTION') and (Result.result <> '') then
      result.error := '';
  end;

  //zsk hodnotu z XML objektu
  function ApiValue(XML: TJclSimpleXML; FName: string): string;
  var
    I: integer;
  begin
    result := '';
    //pokud nen root callback, mm smlu a patnej soubor
    if XML.Root.Name <> 'callback' then exit;
    //pokud jsem nedostal nic
    if XML.Root.Items.count < 1 then exit;
    //pro kadej prvek XML souboru
    for i  := 0 to XML.Root.Items.count - 1 do
    begin
      //s polokou
      with XML.Root.Items[i] do
      begin
        if Name = Fname then Result := Value;
      end;
    end;
  end;


  //porv platnost tokenu
  function ApiAuth(email, token: string; quiet: boolean): boolean;
  var
    params: string;
    R: TJclSimpleXML;
    V: TApiResult;
  begin
    params := '?email=' + EncodeURLElement(email) + '&token=' + token;

    R := ApiRequest('auth', params);
    V := ApiResult(R);

    if V.Result <> '' then
    begin
      MainSettings.SID := ApiValue(R, 'SID');
      if MainSettings.SID <> '' then result := true else result := false;
    end;

    if V.Error <> '' then
    begin
      if not quiet then ShowMessage(TranslateAPIError(V.Error));
      result := false;
      MainSettings.SID := '';
    end;

    R.Free;
  end;



  //zsk (a ulo do nastaven) token
  procedure ApiLogin(Email, Password: string);
  var
    params: string;
    R: TJclSimpleXML;
    V: TApiResult;
  begin
    params :=         '?email=';
    params := params + EncodeURLElement(Email);
    params := params + '&password=';
    params := params + EncodeURLElement(Password);
    params := params + '&nick=';
    params := params + EncodeURLElement(gsUsername);

    R := ApiRequest('login', params);
    V := ApiResult(R);

    MainSettings.token := '';
    MainSettings.SID := '';

    if V.error <> '' then ShowMessage(TranslateAPIError(V.Error));
    if V.result = 'OK_LOG' then
    begin
      LDH('Logged in!');
      MainSettings.Token := ApiValue(R, 'token');
      LDH('Token: ' + MainSettings.Token);
      ShowMessage(_('Logged in successfully!'));
    end;
    R.Free;
  end;

  //Registrace
  procedure ApiRegister(Email, Password: string; Agree: boolean);
  var
    params: string;
    R: TJclSimpleXML;
    V: TApiResult;
  begin
    params :=         '?email=';
    params := params + EncodeURLElement(Email);
    params := params + '&password=';
    params := params + EncodeURLElement(Password);
    params := params + '&nick=';
    params := params + EncodeURLElement(gsUsername);
    if agree then params := params + '&souhlas=1';

    R := ApiRequest('register', params);
    V := ApiResult(R);

    MainSettings.token := '';
    MainSettings.SID := '';

    if V.error <> '' then ShowMessage(TranslateAPIError(V.Error));
    if V.result = 'OK_REG' then
    begin
      LDH('Registered!');
      MainSettings.Token := ApiValue(R, 'token');
      LDH('Token: ' + MainSettings.Token);
      ShowMessage(_('Registered successfully!'));
    end;
    R.Free;
  end;

  procedure ApiChangePassword(email: string);
  var
    params: string;
    R: TJclSimpleXML;
    V: TApiResult;
  begin
    params :=         '?email=';
    params := params + EncodeURLElement(Email);
    params := params + '&nick=';
    params := params + EncodeURLElement(gsUsername);

    R := ApiRequest('change_password', params);
    V := ApiResult(R);

    if V.error <> '' then ShowMessage(TranslateAPIError(V.Error));
    if V.result = 'OK_CHANGE_REQUEST' then
    begin
      ShowMessage(_('Request sent. Please look into yours mailbox, there should be an email with instructions.'));
    end;
    R.Free;
  end;

  function ApiUpload(filename: string): string;
  var
    content: string;
    xmlString: string;
    params: string;
    R: TJCLSimpleXML;
    V: TApiResult;
  begin
    LDH('Filename: ' + filename);

    //zkusm soubor
    if not FileExists(filename) then
    begin
      Result := 'ERR_NO_FILE';
      exit;
    end;
    //natu si soubor
    content := FileToString(filename);
    //zakduju
    content := EncodeBase64(content);

    //zskm jednoduch nzev souboru, abych nepedval celou cestu
    While Pos('\', 'salt' + filename) > 0 do
    begin
      Fetch(filename, '\');
      LDH(Filename);
    end;

    //parametry uploadu
    params :=         '?format=xml';
    params := params + '&filename=';
    params := params + encodeURLElement(filename);
    params := params + '&' + MainSettings.SID;

    //upload
    HttpAsk('POST', 'http://am.valicek.name/upload/' + params, 'application/x-www-form-urlencoded', 'data=' + content, xmlString);

    //vytvom xml soubor
    R := TJclSimpleXML.Create;

    //pozor na chyby bez xml zabalen
    if (xmlString = 'ERR_BAD_FORMAT') or (xmlString = '') or (xmlString = 'ERR_METHOD') then exit;
    //vypu XML string
    LDH(xmlString);
    try
      R.LoadFromString(xmlString);
      LDH('ROOT: ' + R.Root.Name);
      V := ApiResult(R);
      //error
      if V.Error <> '' then
      begin
        Result := V.Error;
        R.Free;
        Exit;
      end;
      if V.Result = 'UPLOAD_OK' then result := ApiValue(R, 'url');
    finally
      //free probhne mimo tuto funkci
    end;
    R.Free;
  end;

end.